/* 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */
/** 
	 * FullMouseMode.java
	 * Autor: Alberto Poblacin Navarro
	 * Aadido al proyecto en la version v0.1
	 * Aadido a:
	 		* Proyecto original: http://code.google.com/p/android-vnc-viewer/
	 		* Release: 39
	 */
package android.androidVNC;

import android.os.CountDownTimer;
import android.view.Display;
import android.view.KeyEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.ImageButton;
import android.widget.RelativeLayout;
import android.widget.Toast;
import android.widget.ZoomControls;
import android.widget.RelativeLayout.LayoutParams;


	/**
	 * Modo para un control global del raton y el teclado remotos
	 * @author Alberto Poblacin Navarro
	 *
	 */
	class FullMouseMode implements AbstractInputHandler {
	
		VncCanvas vncCanvas;
		VncCanvasActivity vncCanvasActivity;
		//Al ser static se pueden utilizar en los demas modos de visualizacion aunque no se haya utilizado el FullMouseMode
		static ZoomControls zoom_control;
		static ImageButton reset;
		/**
		 *  Modo para un control global del ratón y el teclado remotos
		 *  @param vnccanvas instancia de la clase VncCanvas que se está utilizando, es la imagen que se dibuja
		 *  @param vnccanvasactivity instacia de la clase VncCanvasActivity, la encargada de intercambiar informacion entre el rfb protocol y el vnccanvas
		 */
		FullMouseMode(VncCanvas vnccanvas, VncCanvasActivity vnccanvasactivity)
		{
			vncCanvas = vnccanvas;
			vncCanvasActivity = vnccanvasactivity;
			
			//Modo de situacion de los widget en la pantalla: Modo relativo
			RelativeLayout tabla = new RelativeLayout(vncCanvasActivity);
			//Configuramos los parametros de la tabla: que ocupe toda la pantalla
			tabla.setLayoutParams(new LayoutParams(LayoutParams.FILL_PARENT ,LayoutParams.FILL_PARENT));
			zoom_control = new ZoomControls(vncCanvasActivity);
			//Lo inicializamo "escondido" para que solo se muestren cuando se active la opcion de menu
			zoom_control.hide();
			//Configuracion de los parametros d los controles de zoom: ocupen su tamao tal cual
			LayoutParams parametros = new RelativeLayout.LayoutParams(LayoutParams.WRAP_CONTENT, LayoutParams.WRAP_CONTENT);
			//Aadimos configuraciones: alieneado al fondo de padre(table) y centrado horizontalmente
			parametros.addRule(RelativeLayout.ALIGN_PARENT_BOTTOM);
			parametros.addRule(RelativeLayout.CENTER_HORIZONTAL);
			//Aadimos las configuraciones al zoom y configuramos ambos botones.
			zoom_control.setLayoutParams(parametros);
			zoom_control.setOnZoomInClickListener(new OnClickListener(){
				@Override
				public void onClick(View v) {
					Zoom.zoom_in(vncCanvas, vncCanvasActivity);
				}
			});
			zoom_control.setOnZoomOutClickListener(new OnClickListener(){
				@Override
				public void onClick(View v) {
					Zoom.zoom_out(vncCanvas, vncCanvasActivity);
				}
			});
			tabla.addView(zoom_control);
			reset= new ImageButton(vncCanvasActivity);
			//reset.setAlpha(75);
			reset.setImageResource(R.drawable.reset);
			//reset.setBackgroundResource(R.drawable.reset);
			reset.setVisibility(View.INVISIBLE);
			reset.setOnClickListener(new OnClickListener(){
				@Override
				public void onClick(View v) {
					Zoom.reset(vncCanvas, vncCanvasActivity);	
				}
			});
			tabla.addView(reset);
			vncCanvasActivity.addContentView(tabla, tabla.getLayoutParams());
		}
		

		
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onKeyDown(int, android.view.KeyEvent)
		 */
		@Override
		public boolean onKeyDown(int keyCode, KeyEvent evt) {
			switch (keyCode) {
				case KeyEvent.KEYCODE_VOLUME_UP: //Hacemos zoom in
					Zoom.zoom_in(vncCanvas, vncCanvasActivity); 
					break;
				case KeyEvent.KEYCODE_VOLUME_DOWN: //Hacemos zoom out
					Zoom.zoom_out(vncCanvas, vncCanvasActivity); 
					break;
				case KeyEvent.KEYCODE_BACK: //Reseteamos la posicion del reset
					Zoom.reset(vncCanvas, vncCanvasActivity);
					break;
				case  KeyEvent.KEYCODE_DPAD_CENTER: //Mandamos el click derecho del ratn
					KeyEvent camera = new KeyEvent(KeyEvent.ACTION_DOWN,KeyEvent.KEYCODE_CAMERA);
					vncCanvas.processLocalKeyEvent(KeyEvent.KEYCODE_CAMERA, camera);
					vncCanvas.processPointerEvent(MotionEvent.obtain(1, System.currentTimeMillis(), MotionEvent.ACTION_DOWN, vncCanvas.mouseX,vncCanvas.mouseY, 0), false);
					break;
				case KeyEvent.KEYCODE_CALL: //Cremos un nuevo screenshot
					Screenshot.nuevo(vncCanvasActivity, vncCanvas);
					break;
				default:
					vncCanvasActivity.defaultKeyDownHandler( keyCode, evt);
					break;
				}
				return true;
			}
		
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onKeyUp(int, android.view.KeyEvent)
		 */
		
		@Override
		public boolean onKeyUp(int keyCode, KeyEvent evt) {
			
			if(keyCode == KeyEvent.KEYCODE_DPAD_CENTER){ //Click derecho del raton
				vncCanvas.processPointerEvent(MotionEvent.obtain(1, System.currentTimeMillis(), MotionEvent.ACTION_UP, vncCanvas.mouseX,vncCanvas.mouseY, 0), false);
				KeyEvent camera = new KeyEvent(KeyEvent.ACTION_UP,KeyEvent.KEYCODE_CAMERA);
				vncCanvas.processLocalKeyEvent(KeyEvent.KEYCODE_CAMERA, camera);		
				}
			else{
				vncCanvasActivity.defaultKeyDownHandler( keyCode, evt);
			}
				return true;
		}
	
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onTouchEvent(android.view.MotionEvent)
		 */
		@Override
		public boolean onTouchEvent(MotionEvent event) {
			return fullmousetouch(event);			
		}
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#onTrackballEvent(android.view.MotionEvent)
		 */
		@Override
		public boolean onTrackballEvent(MotionEvent evt) {
			return false;
		}
		/* (non-Javadoc)
		 * @see android.androidVNC.AbstractInputHandler#handlerDescription()
		 */
		@Override
		public CharSequence getHandlerDescription() {
			return vncCanvasActivity.getResources().getText(R.string.input_mode_full_mouse);
		}

        /* (non-Javadoc)
         * @see android.androidVNC.AbstractInputHandler#getName()
         */
        @Override
        public String getName() {
                return "FULL MOUSE";
        }
		
	
	private long down; //Momento de la pulsacion
	private long up; // Momento en el que se suelta la pulsacion
	private final int click = 100; //Tiempo en milisegundos para saber que se trata de un click
	private final int drag = 500; //Tiempo en milisegundos para saber si hay que arrastrar en vez de mover
		
	CountDownTimer timer; //Temporizador para saber si se trata de un click o para arrastrar
	
	/**
	 * Segun el parametro event, realiza una accin que se ser enviada al servidor como accin de ratn.
	 * Depende de la accin podemos mandar un movimiento, un click o la accin de arrastrar
	 * @param event evento tctil que sera tratado por la funcin
	 * @return 
	 * @author Alberto Poblacin Navarro
	 */
	boolean fullmousetouch( MotionEvent event) 
	{ 
		//Ajustamos la posicion del evento para que sea compatible con el el Zoom
		event.setLocation((event.getX()/Zoom.zoom), (event.getY()/Zoom.zoom));
		
		switch (event.getAction()) {
			case MotionEvent.ACTION_DOWN:
				//down=new Date();
				down= event.getEventTime(); //Momento de la pulsacion
				final MotionEvent evento = MotionEvent.obtain(event);
				//Si es una pulsacion larga: modo de arrastre. 
				timer = new CountDownTimer(drag, 10){
	
				     public void onTick(long millisUntilFinished) {
				    	 //No hacer nada durante los intervalos
				     }
				     				 
				     public void onFinish() {
				    	 // Si se completa la cuenta atras entramos en el modo de arrastre "Draggin mode"
				    	 Toast.makeText(vncCanvasActivity, "Dragging Mode", Toast.LENGTH_SHORT).show();
				    	 //Mandamos un click down ajustando la posicion de raton segun el zoom
						 evento.setAction(MotionEvent.ACTION_DOWN);
						 // Adjust coordinates for Android notification bar.
						 evento.offsetLocation(0, -1f * vncCanvas.getTop()/Zoom.zoom);
						 // Adjust coordinates for panning position.
						 evento.offsetLocation(vncCanvasActivity.absoluteXPosition/Zoom.zoom, vncCanvasActivity.absoluteYPosition/Zoom.zoom);
						 vncCanvas.processPointerEvent(evento, true);
					}
				  }.start();
				return true;
				
			case MotionEvent.ACTION_MOVE:
				//Cancelas el timeout para que no entre en el modo de arrastre
				timer.cancel();
				return Move(event);
				
				
				
			case MotionEvent.ACTION_UP:
				//up=new Date(); 
				up = event.getEventTime(); //Para conocer la duracion de la pulsacion
				timer.cancel();
				//Si es una pulsacion rapida: click de raton
				if (up - down < click)
				{
					//Mandas el gesto del click complento, cuando pulsas y cuando sueltas la tecla
					event.setAction(MotionEvent.ACTION_DOWN);
					// Adjust coordinates for Android notification bar.
					event.offsetLocation(0, -1f * vncCanvas.getTop()/Zoom.zoom);
					// Adjust coordinates for panning position.
					event.offsetLocation(vncCanvasActivity.absoluteXPosition/Zoom.zoom, vncCanvasActivity.absoluteYPosition/Zoom.zoom);
					//Mandas el click completo: pulsas y sueltas
					vncCanvas.processPointerEvent(event, true);
					event.setAction(MotionEvent.ACTION_UP);
					return vncCanvas.processPointerEvent(event, true);
				}
				//Si no ha sido un click del raton mandas que se levantado la tecla simplemente
				event.setAction(MotionEvent.ACTION_UP);
				// Adjust coordinates for Android notification bar.
				event.offsetLocation(0, -1f * vncCanvas.getTop()/Zoom.zoom);
				// Adjust coordinates for panning position.
				event.offsetLocation(vncCanvasActivity.absoluteXPosition/Zoom.zoom, vncCanvasActivity.absoluteYPosition/Zoom.zoom);
				return vncCanvas.processPointerEvent(event, true);			
			}
		return false;
					
	}
	
	final int border = 30; //Distancia entre el cursor y el borde para que se desplace la pantalla
	/**
	 * Mueve la parte de escritorio en la que nos encontramos si estamos a una distancia menor de X de un extremo
	 * @param event
	 * @author Alberto Poblacion Navarro
	 */
	private boolean Move(MotionEvent event) {
			//Obtienes las cordenadas donde tienes el cursor
			float curX = event.getX();
			float curY = event.getY();
			int scrollX =0;
			int scrollY =0;
			//Si pulsamos el raton a menos distancia de "border" de uno de los extremos, configuramos la posicion que debemos mover la imagen
			//De la actividad conseguimos el manager de la ventana y de este la pantalla por defecto
			Display display=vncCanvasActivity.getWindowManager().getDefaultDisplay();
			if (curX*Zoom.zoom  > (display.getWidth()-border)) scrollX +=border;
			if (curX*Zoom.zoom  < border) scrollX -=border;
			if (curY*Zoom.zoom  > (display.getHeight()-border)) scrollY +=border;
			if (curY*Zoom.zoom  < border) scrollY -=border;
			//Toast.makeText(vncCanvasActivity, "CurX: "+curX+" CurY: "+curY,Toast.LENGTH_SHORT).show();
			//Otro mtodo de obtener los bordes, pero el borde superior e inferior no funcionan del todo bien
			/*switch (event.getEdgeFlags()) {
			case MotionEvent.EDGE_LEFT:
				scrollX -=border;
				break;
			case MotionEvent.EDGE_RIGHT:
				scrollX +=border;
				break;
			case MotionEvent.EDGE_TOP:
				scrollY -=border;
				break;
			case MotionEvent.EDGE_BOTTOM:
				scrollY +=border;
				break;
			}*/
			//Sumamos a la posicin absoluta la cantidad que no tengamos que mover	
			vncCanvasActivity.absoluteXPosition += scrollX;
			vncCanvasActivity.absoluteYPosition += scrollY;
			//Comprobamos los margenes superior e izquierdo
			if(vncCanvasActivity.absoluteXPosition < 0){
				scrollX=0;
				vncCanvasActivity.absoluteXPosition=0;
			}
			if(vncCanvasActivity.absoluteYPosition < 0){
				scrollY=0;
				vncCanvasActivity.absoluteYPosition=0;
			}
			//Comprobamos los mrgenes derecho y inferior 
			if (vncCanvasActivity.absoluteXPosition + vncCanvas.getWidth()> vncCanvas.getImageWidth()){
				scrollX=0;
				vncCanvasActivity.absoluteXPosition-=border;
			}
			if (vncCanvasActivity.absoluteYPosition + vncCanvas.getHeight() > vncCanvas.getImageHeight()){
				scrollY=0;
				vncCanvasActivity.absoluteYPosition-=border;
			}
			//Movemos la posicion del escritorio
			vncCanvas.scrollBy(scrollX, scrollY);
			//Ajustamos la posicion del evento la barra de notificaciones
			event.offsetLocation(0, -1f * vncCanvas.getTop()/Zoom.zoom);
			//Ajustamos la posicion del evento lo que nos movamos
			event.offsetLocation(vncCanvasActivity.absoluteXPosition/Zoom.zoom,vncCanvasActivity.absoluteYPosition/Zoom.zoom);
			//Procesamos el evento sin el click pulsado, solo movimiento de raton
			return vncCanvas.processPointerEvent(event, false);
	}
}